/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_InspectorUtilsBinding_h__
#define mozilla_dom_InspectorUtilsBinding_h__

#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/DOMJSClass.h"
#include "mozilla/dom/DOMJSProxyHandler.h"

namespace mozilla {
namespace dom {

struct InspectorRGBTriple : public MainThreadDictionaryBase {
  InspectorRGBTriple() {}
  bool Init(JSContext* cx, JS::Handle<JS::Value> val);
  bool Init(const nsAString& aJSON);
  bool ToObject(JSContext* cx, JS::Handle<JSObject*> parentObject, JS::Value *vp) const;
  void TraceDictionary(JSTracer* trc);

  uint8_t mB;
  uint8_t mG;
  uint8_t mR;
private:
  // Disallow copy-construction
  InspectorRGBTriple(const InspectorRGBTriple&) MOZ_DELETE;
  static bool InitIds(JSContext* cx);
  static bool initedIds;
  static jsid b_id;
  static jsid g_id;
  static jsid r_id;
};
struct InspectorRGBTripleInitializer : public InspectorRGBTriple {
  InspectorRGBTripleInitializer() {
    // Safe to pass a null context if we pass a null value
    Init(nullptr, JS::NullHandleValue);
  }
};

} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_InspectorUtilsBinding_h__
