/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

[scriptable, builtinclass, uuid(d26880c8-0468-4abb-94a6-9143defb5545)]
interface nsIDOMUSSDReceivedEvent : nsIDOMEvent
{
  readonly attribute DOMString message;
  [infallible] readonly attribute boolean sessionEnded;

  [noscript] void initUSSDReceivedEvent(in DOMString aType,
                                        in boolean aCanBubble,
                                        in boolean aCancelable,
                                        in DOMString aMessage,
                                        in boolean aSessionEnded);
};

dictionary USSDReceivedEventInit : EventInit
{
  DOMString message;
  boolean sessionEnded;
};
