/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/* mfbt math constants. */

#ifndef mozilla_Constants_h_
#define mozilla_Constants_h_

#ifndef M_PI
#  define M_PI 3.14159265358979323846
#endif

#endif  /* mozilla_Constants_h_ */
