/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* vim: set sw=4 ts=4 et tw=80 : */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * nsIRemoteOpenFileListener: passed to RemoteOpenFileChild::AsyncRemoteFileOpen.
 *
 * Interface for notifying when the file has been opened and is available in
 * child.
 */
[uuid(5c89208c-fe2b-4e04-9783-93bcf5c3b783)]
interface nsIRemoteOpenFileListener : nsISupports
{
    /**
     * Called when result of opening RemoteOpenFileChild:AsyncRemoteFileOpen()
     * is available in child.
     *
     * @param aOpenStatus: nsresult from opening file in parent.  If NS_OK,
     * then a following call to RemoteOpenFileChild::OpenNSPRFileDesc that
     * passes the same flags as were passed to
     * RemoteOpenFileChild::AsyncRemoteFileOpen is guaranteed to succeed.  If
     * !NS_OK or if different flags were passed, the call will fail.
     */
    void onRemoteFileOpenComplete(in nsresult aOpenStatus);
};

