/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/* The general dialog posing function within nsPromptService, for
   private consumption, only. */

#include "nsISupports.idl"

interface nsIDOMWindow;
interface nsIDialogParamBlock;

[uuid(C60A1955-6CB3-4827-8EF8-4F5C668AF0B3)]
interface nsPIPromptService : nsISupports
{
%{C++
  // eOpeningSound is obsolete but we need to support it for the compatibility.
  // The implementers should use eSoundEventId instead.
  enum {eMsg=0, eCheckboxMsg=1, eIconClass=2, eTitleMessage=3, eEditfield1Msg=4,
        eEditfield2Msg=5, eEditfield1Value=6, eEditfield2Value=7,
        eButton0Text=8, eButton1Text=9, eButton2Text=10, eButton3Text=11,
        eDialogTitle=12, eOpeningSound=13};
  enum {eButtonPressed=0, eCheckboxState=1, eNumberButtons=2,
        eNumberEditfields=3, eEditField1Password=4, eDefaultButton=5,
        eDelayButtonEnable=6, eSoundEventId=7};
%}

   void doDialog(in nsIDOMWindow aParent, in nsIDialogParamBlock aParamBlock, in string aChromeURL);
};

